@@ -23,6 +23,49 @@
  * SPDX-License-Identifier: LGPL-2.1-only
  */
 
+/**
+ * CasildaCompositor:
+ *
+ * A simple Wayland compositor widget for Gtk 4.
+ *
+ * It was originally created for Cambalache's workspace using (wlroots)[https://gitlab.freedesktop.org/wlroots/wlroots],
+ * a modular library to create Wayland compositors.
+ *
+ * Following Wayland tradition, this library is named after my hometown in Santa Fe, Argentina.
+ *
+ * To embed another process window in your Gtk 4 application all you have to do is create a CasildaCompositor widget
+ * and add it in the hierarchy just like any other widget.
+ *
+ * You can specify which UNIX socket the compositor will listen for clients connections or use
+ * [method@Casilda.Compositor.get_client_socket_fd] to get a already connected socket to the compositor.
+ *
+ * ```c
+ * compositor = casilda_compositor_new ("/tmp/casilda-example.sock");
+ * gtk_window_set_child (GTK_WINDOW (window), GTK_WIDGET (compositor));
+ * ```
+ *
+ * Once the compositor is running you can connect to it by specifying the socket in WAYLAND_DISPLAY environment variable.
+ *
+ * ```bash
+ * export GDK_BACKEND=wayland
+ * export WAYLAND_DISPLAY=/tmp/casilda-example.sock
+ * gtk4-demo
+ * ```
+ *
+ * If you do not want any client being able to connect to the compositor you can pass NULL as socket and spawn the client
+ * with [method@Casilda.Compositor.spawn_async] or get an already connected socket with
+ * [method@Casilda.Compositor.get_client_socket_fd] and pass it to the client on WAYLAND_SOCKET env variable.
+ *
+ * ```c
+ * compositor = casilda_compositor_new (NULL);
+ * gtk_window_set_child (GTK_WINDOW (window), GTK_WIDGET (compositor));
+ * gtk_window_present (GTK_WINDOW (window));
+ *
+ * gchar *argv[] = { "/usr/bin/gtk4-demo", NULL };
+ * casilda_compositor_spawn_async (compositor, NULL, argv, NULL, G_SPAWN_DEFAULT, NULL, NULL, NULL, NULL);
+ * ```
+ */
+
 #define _POSIX_C_SOURCE 199309L /* clock_gettime */
 #define WLR_USE_UNSTABLE 1
 #define G_LOG_DOMAIN "Casilda"
@@ -169,7 +212,6 @@ typedef struct
 
   /* GObject properties */
   gchar       *socket;
-  gboolean     scrollable;
 
   GtkAdjustment *adjustment[2];
 } CasildaCompositorPrivate;
@@ -257,7 +299,6 @@ typedef struct
 enum {
   PROP_0,
   PROP_SOCKET,
-  PROP_SCROLLABLE,
 
   N_PROPERTIES,
 
@@ -591,6 +632,26 @@ casilda_compositor_snapshot_toplevel (CasildaCompositorToplevel *toplevel, Snaps
     casilda_compositor_snapshot_toplevel (l->data, data);
 }
 
+static void
+casilda_compositor_get_scroll_offset(CasildaCompositorPrivate *priv, gint *x, gint *y)
+{
+  if (priv->adjustment[GTK_ORIENTATION_HORIZONTAL])
+    *x = gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_HORIZONTAL]);
+  else
+    *x = 0;
+
+  if (priv->adjustment[GTK_ORIENTATION_VERTICAL])
+    *y = gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_VERTICAL]);
+  else
+    *y = 0;
+}
+
+static inline gboolean
+casilda_compositor_is_scrollable(CasildaCompositorPrivate *priv)
+{
+  return priv->adjustment[GTK_ORIENTATION_HORIZONTAL] && priv->adjustment[GTK_ORIENTATION_VERTICAL];
+}
+
 static void
 casilda_compositor_snapshot (GtkWidget *widget, GtkSnapshot *snapshot)
 {
@@ -602,8 +663,7 @@ casilda_compositor_snapshot (GtkWidget *widget, GtkSnapshot *snapshot)
   clock_gettime (CLOCK_MONOTONIC, &data.now);
 
   /* Translate by scrollable offset */
-  x = gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_HORIZONTAL]);
-  y = gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_VERTICAL]);
+  casilda_compositor_get_scroll_offset(priv, &x, &y);
 
   gtk_snapshot_save (snapshot);
   gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (-x, -y));
@@ -638,9 +698,6 @@ static void compositor_update_adjustment_toplevel (CasildaCompositorPrivate *pri
 static void
 compositor_update_adjustment (CasildaCompositorPrivate *priv, GList *toplevels)
 {
-  if (!priv->scrollable)
-    return;
-
   for (GList *l = toplevels; l; l = g_list_next (l))
     {
       CasildaCompositorToplevel *toplevel = l->data;
@@ -657,7 +714,7 @@ compositor_update_adjustment_toplevel (CasildaCompositorPrivate *priv, CasildaCo
 {
   struct wlr_box tbox = {0, };
 
-  if (!priv->scrollable || !toplevel->xdg_toplevel)
+  if (!casilda_compositor_is_scrollable(priv) || !toplevel->xdg_toplevel)
     return;
 
   wlr_xdg_surface_get_geometry (toplevel->xdg_toplevel->base, &tbox);
@@ -685,14 +742,14 @@ casilda_compositor_size_allocate (GtkWidget *widget, int w, int h, G_GNUC_UNUSED
   w = w * scale;
   h = h * scale;
 
-  gtk_adjustment_set_page_size (priv->adjustment[GTK_ORIENTATION_HORIZONTAL], w);
-  gtk_adjustment_set_page_size (priv->adjustment[GTK_ORIENTATION_VERTICAL], h);
-  gtk_adjustment_set_upper (priv->adjustment[GTK_ORIENTATION_HORIZONTAL], w);
-  gtk_adjustment_set_upper (priv->adjustment[GTK_ORIENTATION_VERTICAL], h);
-
-  /* Update scrollable values */
-  if (priv->scrollable)
+  if (casilda_compositor_is_scrollable(priv))
     {
+      gtk_adjustment_set_page_size (priv->adjustment[GTK_ORIENTATION_HORIZONTAL], w);
+      gtk_adjustment_set_page_size (priv->adjustment[GTK_ORIENTATION_VERTICAL], h);
+      gtk_adjustment_set_upper (priv->adjustment[GTK_ORIENTATION_HORIZONTAL], w);
+      gtk_adjustment_set_upper (priv->adjustment[GTK_ORIENTATION_VERTICAL], h);
+
+      /* Update scrollable values */
       compositor_update_adjustment (priv, priv->toplevels);
 
       w = gtk_adjustment_get_upper (priv->adjustment[GTK_ORIENTATION_HORIZONTAL]);
@@ -1128,8 +1185,12 @@ on_motion_controller_enter (G_GNUC_UNUSED GtkEventControllerMotion *self,
                             gdouble                                 y,
                             CasildaCompositorPrivate               *priv)
 {
-  priv->pointer_x = x + gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_HORIZONTAL]);
-  priv->pointer_y = y + gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_VERTICAL]);
+  gint x_offset, y_offset;
+
+  casilda_compositor_get_scroll_offset (priv, &x_offset, &y_offset);
+
+  priv->pointer_x = x + x_offset;
+  priv->pointer_y = y + y_offset;
   casilda_compositor_handle_pointer_motion (priv);
   wlr_seat_pointer_notify_frame (priv->seat);
 }
@@ -1147,13 +1208,17 @@ on_motion_controller_motion (G_GNUC_UNUSED GtkEventControllerMotion *self,
                              gdouble                                 y,
                              CasildaCompositorPrivate               *priv)
 {
+  gint x_offset, y_offset;
+
+  casilda_compositor_get_scroll_offset (priv, &x_offset, &y_offset);
+
   /* Clamp pointer to widget coordinates */
   x = CLAMP (x, 0, gtk_widget_get_width (priv->self));
   y = CLAMP (y, 0, gtk_widget_get_height (priv->self));
 
   /* Add scrollable offset */
-  priv->pointer_x = x + gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_HORIZONTAL]);
-  priv->pointer_y = y + gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_VERTICAL]);
+  priv->pointer_x = x + x_offset;
+  priv->pointer_y = y + y_offset;
 
   casilda_compositor_handle_pointer_motion (priv);
   wlr_seat_pointer_notify_frame (priv->seat);
@@ -1758,20 +1823,20 @@ compositor_set_adjustment (CasildaCompositor *compositor, GtkOrientation orienta
   CasildaCompositorPrivate *priv = GET_PRIVATE (compositor);
   GtkAdjustment **adjustment_prop = &priv->adjustment[orientation];
 
-  if (adjustment && adjustment == *adjustment_prop)
+  if (adjustment == *adjustment_prop)
     return;
 
   if (*adjustment_prop)
     g_signal_handlers_disconnect_by_func (*adjustment_prop, compositor_adjustment_value_changed, compositor);
 
-  if (!adjustment)
-    adjustment = gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
+  g_set_object (adjustment_prop, adjustment);
 
-  g_set_object (adjustment_prop, g_object_ref_sink (adjustment));
+  if (adjustment)
+    {
+      g_signal_connect (adjustment, "value-changed", G_CALLBACK (compositor_adjustment_value_changed), compositor);
 
-  g_signal_connect (adjustment, "value-changed", G_CALLBACK (compositor_adjustment_value_changed), compositor);
-
-  compositor_adjustment_value_changed (adjustment, compositor);
+      compositor_adjustment_value_changed (adjustment, compositor);
+    }
 }
 
 static void
@@ -1789,9 +1854,6 @@ casilda_compositor_set_property (GObject      *object,
     case PROP_SOCKET:
       g_set_str (&priv->socket, g_value_get_string (value));
       break;
-    case PROP_SCROLLABLE:
-      casilda_compositor_set_scrollable(CASILDA_COMPOSITOR (object), g_value_get_boolean (value));
-      break;
     case PROP_HADJUSTMENT:
       compositor_set_adjustment (CASILDA_COMPOSITOR (object), GTK_ORIENTATION_HORIZONTAL, g_value_get_object (value));
       break;
@@ -1825,9 +1887,6 @@ casilda_compositor_get_property (GObject    *object,
     case PROP_SOCKET:
       g_value_set_string (value, priv->socket);
       break;
-    case PROP_SCROLLABLE:
-      g_value_set_boolean (value, priv->scrollable);
-      break;
     case PROP_HADJUSTMENT:
       g_value_set_object (value, priv->adjustment[GTK_ORIENTATION_HORIZONTAL]);
       break;
@@ -1876,6 +1935,8 @@ casilda_compositor_unrealize (GtkWidget *widget)
       priv->frame_clock_source = 0;
     }
 
+  priv->frame_clock = NULL;
+
   GTK_WIDGET_CLASS (casilda_compositor_parent_class)->unrealize (widget);
 }
 
@@ -1916,16 +1977,6 @@ casilda_compositor_class_init (CasildaCompositorClass *klass)
   properties[PROP_SOCKET] =
     g_param_spec_string ("socket", "", "", NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
-  /**
-   * CasildaCompositor:scrollable:
-   *
-   * True to make widget scrollable. The virtual viewport will grow automatically when a window
-   * is moved out off the viewing area.
-   *
-   */
-  properties[PROP_SCROLLABLE] =
-    g_param_spec_boolean ("scrollable", "", "", FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
-
   g_object_class_install_properties (object_class, N_PROPERTIES, properties);
 }
 
@@ -1949,8 +2000,9 @@ xdg_toplevel_map (struct wl_listener *listener, G_GNUC_UNUSED void *data)
   struct wlr_xdg_toplevel *xdg_toplevel = toplevel->xdg_toplevel;
   CasildaCompositorToplevelState *state = toplevel->state;
   struct wlr_box box;
-  gint offset_x = gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_HORIZONTAL]);
-  gint offset_y = gtk_adjustment_get_value (priv->adjustment[GTK_ORIENTATION_VERTICAL]);
+  gint offset_x, offset_y;
+
+  casilda_compositor_get_scroll_offset (priv, &offset_x, &offset_y);
 
   wlr_xdg_surface_get_geometry (xdg_toplevel->base, &box);
 
@@ -2578,6 +2630,15 @@ casilda_compositor_wlr_init (CasildaCompositorPrivate *priv)
 
 /* Public API */
 
+/**
+ * casilda_compositor_new:
+ * @socket: the named socket to use or NULL
+ *
+ * Create a new `CasildaCompositor` widget using @socket as the name of the Wayland socket.
+ * Clients can connect to the compositor by setting WAYLAND_DISPLAY=@socket
+ *
+ * Returns: a new `CasildaCompositor` widget
+ */
 CasildaCompositor *
 casilda_compositor_new (const gchar *socket)
 {
@@ -2634,8 +2695,8 @@ casilda_compositor_get_client_socket_fd (CasildaCompositor *compositor)
  *
  * Executes a child program asynchronously with the right environment to automatically connect to this compositor.
  *
- * See g_spawn_async_with_pipes_and_fds() for a full description; this function simply calls
- * g_spawn_async_with_pipes_and_fds() without any pipes and with WAYLAND_DISPLAY set to `CasildaCompositor::socket` or
+ * See [func@GLib.spawn_async_with_pipes_and_fds] for a full description; this function simply calls
+ * [func@GLib.spawn_async_with_pipes_and_fds] without any pipes and with WAYLAND_DISPLAY set to `CasildaCompositor::socket` or
  * a fd already connected to the compositor and set to WAYLAND_SOCKET if its NULL.
  *
  * Please note GDK_BACKEND, WAYLAND_DISPLAY and WAYLAND_SOCKET are set by this function they are ignored from %envp and
@@ -2719,26 +2780,3 @@ casilda_compositor_spawn_async (CasildaCompositor *compositor,
   return retval;
 }
 
-
-/**
- * casilda_compositor_set_scrollable:
- * @compositor: a `CasildaCompositor`
- * @scrollable:
- *
- * Flag to control if the compositor should be scrollable or not.
- *
- * When true the compositor will have a virtual viewport that grows automatically when a
- * client window is moved outside of the compositor allocation.
- */
-void
-casilda_compositor_set_scrollable (CasildaCompositor *compositor, gboolean scrollable)
-{
-  g_return_if_fail (CASILDA_IS_COMPOSITOR (compositor));
-
-  CasildaCompositorPrivate *priv = GET_PRIVATE (compositor);
-
-  priv->scrollable = scrollable;
-
-  gtk_widget_queue_resize (GTK_WIDGET (compositor));
-}
-
